﻿using System;
using System.Collections.Generic;

namespace HeuristicLab.Problems.ProgramSynthesis.Push.Configuration {
  using HeuristicLab.Core;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Expressions;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  public class EnabledExpressionsChangedEventArgs : EventArgs {
    public IEnumerable<string> AddedExpressions { get; private set; }
    public IEnumerable<string> RemovedExpressions { get; private set; }

    public EnabledExpressionsChangedEventArgs(IEnumerable<string> addedExpressions, IEnumerable<string> removedExpressions) {
      AddedExpressions = addedExpressions;
      RemovedExpressions = removedExpressions;
    }
  }

  public interface IEnabledExpressionsConfiguration : INamedItem {
    event EventHandler<EnabledExpressionsChangedEventArgs> EnabledExpressionsChanged;

    IList<string> EnabledExpressions { get; }

    void EnableStack(StackTypes types, bool enableExpressions = false);
    void DisableStack(StackTypes type, bool disableExpressions = false);
    void SetStack(StackTypes type, bool state, bool cascadeForExpressions = false);
    void EnableExpressionOfStack(StackTypes types);
    void DisableExpressionOfStack(StackTypes types);
    void EnableExpression(string name, bool enableStackIfDisabled = false);
    void DisableExpression(string name, bool disableStackIfEnabled = false);
    void SetExpression(string name, bool state, bool cascadeForStack = false);
    void SetExpression<T>(bool state, bool cascadeForStack = false) where T : Expression;
    void EnableExpression<T>(bool enableStackIfDisabled = false) where T : Expression;
    void DisableExpression<T>(bool disableStackIfEnabled = false) where T : Expression;
  }
}
