﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Erc {

  using HeuristicLab.Common;
  using HeuristicLab.Core;
  using HeuristicLab.Data;
  using HeuristicLab.Parameters;
  using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Erc.Interfaces;

  [StorableClass]
  public class NameErcOptions : StringErcOptions, IReadOnlyNameErcOptions {
    private const string NewNameProbabilityParameterName = "New name probability";

    public NameErcOptions() {
      Parameters.Add(new FixedValueParameter<PercentValue>(NewNameProbabilityParameterName));
    }

    [StorableConstructor]
    public NameErcOptions(bool deserializing) : base(deserializing) { }

    public NameErcOptions(NameErcOptions origin, Cloner cloner) : base(origin, cloner) { }

    /// <summary>
    ///     The probability that the selection of the ephemeral
    ///     random NAME constant for inclusion in randomly generated code will produce a new name
    ///     (rather than a name that was previously generated).
    /// </summary>
    public IValueParameter<PercentValue> NewNameProbabilityParameter
    {
      get { return (IValueParameter<PercentValue>)Parameters[NewNameProbabilityParameterName]; }
    }

    public double NewNameProbability
    {
      get { return NewNameProbabilityParameter.Value.Value; }
      set { NewNameProbabilityParameter.Value.Value = value; }
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new NameErcOptions(this, cloner);
    }
  }
}
