﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using System;

  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  [PushExpression(StackTypes.Char, "CHAR.ISWHITESPACE", StackTypes.Boolean)]
  public class CharIsWhitespaceExpression : StatelessExpression {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      if (interpreter.CharStack.IsEmpty) return false;

      var c = interpreter.CharStack.Pop();
      interpreter.BooleanStack.Push(char.IsWhiteSpace(c));
      return true;
    }
  }

  [PushExpression(StackTypes.Char, "CHAR.ISLETTER", StackTypes.Boolean)]
  public class CharIsLetterExpression : StatelessExpression {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      if (interpreter.CharStack.IsEmpty) return false;

      var c = interpreter.CharStack.Pop();
      interpreter.BooleanStack.Push(char.IsLetter(c));
      return true;
    }
  }

  [PushExpression(StackTypes.Char, "CHAR.ISDIGIT", StackTypes.Boolean)]
  public class CharIsDigitExpression : StatelessExpression {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      if (interpreter.CharStack.IsEmpty) return false;

      var c = interpreter.CharStack.Pop();
      interpreter.BooleanStack.Push(char.IsDigit(c));
      return true;
    }
  }

  [PushExpression(StackTypes.Char, "CHAR.FROMINTEGER", StackTypes.Integer)]
  public class CharFromIntegerExpression : StatelessExpression {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      if (interpreter.IntegerStack.IsEmpty) return false;

      var value = (int)Math.Abs(interpreter.IntegerStack.Pop() % 128);
      var c = Convert.ToChar(value);

      interpreter.CharStack.Push(c);
      return true;
    }
  }

  [PushExpression(StackTypes.Char, "CHAR.FROMFLOAT", StackTypes.Float)]
  public class CharFromFloatExpression : StatelessExpression {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      if (interpreter.FloatStack.IsEmpty) return false;

      var value = (int)Math.Abs(interpreter.FloatStack.Pop() % 128);
      var c = Convert.ToChar(value);

      interpreter.CharStack.Push(c);
      return true;
    }
  }

  [PushExpression(StackTypes.Char, "CHAR.ALLFROMSTRING", StackTypes.String)]
  public class CharAllFromStringExpression : StatelessExpression {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      if (interpreter.StringStack.IsEmpty) return false;

      var chars = interpreter.StringStack.Pop().ToCharArray();
      interpreter.CharStack.Push(chars);
      return true;
    }
  }

}
