﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using System.Collections.Generic;

  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;

  using Stack;

  /// <summary>
  /// Tells whether that stack is empty.
  /// </summary>
  public abstract class EmptyExpresssion<T> : StatelessExpression {
    public bool Eval(IPushStack<T> stack, IPushStack<bool> booleanStack) {
      booleanStack.Push(stack.IsEmpty);

      return true;
    }
  }

  [PushExpression(StackTypes.Exec, "EXEC.EMPTY", StackTypes.Boolean)]
  public class ExecEmptyExpression : EmptyExpresssion<Expression> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.ExecStack, interpreter.BooleanStack);
    }
  }

  [PushExpression(StackTypes.Code, "CODE.EMPTY", StackTypes.Boolean)]
  public class CodeEmptyExpression : EmptyExpresssion<Expression> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.CodeStack, interpreter.BooleanStack);
    }
  }

  [PushExpression(StackTypes.Integer, "INTEGER.EMPTY", StackTypes.Boolean)]
  public class IntegerEmptyExpression : EmptyExpresssion<long> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.IntegerStack, interpreter.BooleanStack);
    }
  }

  [PushExpression(StackTypes.Float, "FLOAT.EMPTY", StackTypes.Boolean)]
  public class FloatEmptyExpression : EmptyExpresssion<double> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.FloatStack, interpreter.BooleanStack);
    }
  }

  [PushExpression(StackTypes.Boolean, "BOOLEAN.EMPTY", StackTypes.Boolean)]
  public class BooleanEmptyExpression : EmptyExpresssion<bool> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.BooleanStack, interpreter.BooleanStack);
    }
  }

  [PushExpression(StackTypes.Char, "CHAR.EMPTY", StackTypes.Boolean)]
  public class CharEmptyExpression : EmptyExpresssion<char> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.CharStack, interpreter.BooleanStack);
    }
  }

  [PushExpression(StackTypes.String, "STRING.EMPTY", StackTypes.Boolean)]
  public class StringEmptyExpression : EmptyExpresssion<string> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.StringStack, interpreter.BooleanStack);
    }
  }

  [PushExpression(StackTypes.IntegerVector, "INTEGER[].EMPTY", StackTypes.Boolean)]
  public class IntegerVectorEmptyExpression : EmptyExpresssion<List<long>> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.IntegerVectorStack, interpreter.BooleanStack);
    }
  }
}
