﻿using System.Collections.Generic;

namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  /// <summary>
  /// Finds the index of the top lit-type item in the top type vector.
  /// </summary>
  /// <typeparam name="T"></typeparam>
  public abstract class VectorIndexOfExpression<T> : StatelessExpression {
    protected bool Eval(
      IInternalPushInterpreter interpreter,
      IPushStack<List<T>> vectorStack,
      IPushStack<T> literalStack,
      bool canOverride = false) {
      if (vectorStack.IsEmpty || literalStack.IsEmpty)
        return false;

      var vector = vectorStack.Pop();
      var literal = canOverride ? literalStack.Top : literalStack.Pop();
      var result = vector.IndexOf(literal);

      if (canOverride) interpreter.IntegerStack.SetTop(result);
      else interpreter.IntegerStack.Push(result);
      return true;
    }
  }

  [PushExpression(StackTypes.IntegerVector, "INTEGER[].INDEXOF", StackTypes.Integer)]
  public class IntegerVectorIndexOfExpression : VectorIndexOfExpression<long> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.IntegerVectorStack, interpreter.IntegerStack, true);
    }
  }

  [PushExpression(StackTypes.FloatVector, "FLOAT[].INDEXOF", StackTypes.Float | StackTypes.Integer)]
  public class FloatVectorIndexOfExpression : VectorIndexOfExpression<double> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.FloatVectorStack, interpreter.FloatStack);
    }
  }

  [PushExpression(StackTypes.BooleanVector, "BOOLEAN[].INDEXOF", StackTypes.Boolean | StackTypes.Integer)]
  public class BooleanVectorIndexOfExpression : VectorIndexOfExpression<bool> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.BooleanVectorStack, interpreter.BooleanStack);
    }
  }

  [PushExpression(StackTypes.StringVector, "STRING[].INDEXOF", StackTypes.String | StackTypes.String)]
  public class StringVectorIndexOfExpression : VectorIndexOfExpression<string> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.StringVectorStack, interpreter.StringStack);
    }
  }
}
