﻿using System.Collections.Generic;

namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  /// <summary>
  /// Replace first occurrences of the second lit-type item with the first lit-type item in the top type vector.
  /// </summary>
  /// <typeparam name="T"></typeparam>
  public abstract class VectorReplaceFirstExpression<T> : StatelessExpression {
    protected bool Eval(IPushStack<List<T>> vectorStack, IPushStack<T> literalStack) {
      if (vectorStack.IsEmpty ||
          literalStack.Count < 2)
        return false;

      var vector = vectorStack.Top;
      var first = literalStack.Top;
      var second = literalStack.ReverseElementAt(1);
      var index = vector.IndexOf(second);
      literalStack.Remove(2);

      if (index >= 0)
        vector[index] = first;

      return true;
    }
  }

  [PushExpression(StackTypes.IntegerVector, "INTEGER[].REPLACEFIRST", StackTypes.Integer)]
  public class IntegerVectorReplaceFirstExpression : VectorReplaceFirstExpression<long> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.IntegerVectorStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackTypes.FloatVector, "FLOAT[].REPLACEFIRST", StackTypes.Float)]
  public class FloatVectorReplaceFirstExpression : VectorReplaceFirstExpression<double> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.FloatVectorStack, interpreter.FloatStack);
    }
  }

  [PushExpression(StackTypes.BooleanVector, "BOOLEAN[].REPLACEFIRST", StackTypes.Boolean)]
  public class BooleanVectorReplaceFirstExpression : VectorReplaceFirstExpression<bool> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.BooleanVectorStack, interpreter.BooleanStack);
    }
  }

  [PushExpression(StackTypes.StringVector, "STRING[].REPLACEFIRST", StackTypes.String)]
  public class StringVectorReplaceFirstExpression : VectorReplaceFirstExpression<string> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.StringVectorStack, interpreter.StringStack);
    }
  }
}
